/* 
 * (c) 2014 - 2020 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include <string.h>
#include "color.h"
#include "draw-funcs.h"
#include "dibuja.h"
#include "util.h"
#include "dibuja-colorsel.h"
#include "palette-color.h"


#ifndef MOUSE_RIGHT_BUTTON
	#define  MOUSE_RIGHT_BUTTON	(3)
#endif
#ifndef MOUSE_LEFT_BUTTON
	#define  MOUSE_LEFT_BUTTON	(1)
#endif

#if GTK_CHECK_VERSION(3, 4, 0)
	#define NOTIFY_COLOR "notify::rgba"
#else
	#define NOTIFY_COLOR "notify::color"
#endif


static void tool_color_button_press_event(PaletteColor *pc, gint button);


static const gchar *default_colors[N_BUTTONS] = {
	       "black",      "white",     "gray50",         "red",      "purple",
	        "blue", "light blue",      "green",      "yellow",      "orange",
	    "lavender",      "brown", "goldenrod4", "dodger blue",        "pink",
	 "light green",     "gray10",     "gray30",      "gray75",      "gray90"
};

void color_sccolor2rgba(SCColor *color,
						guchar *r, guchar *g, guchar *b, guchar *a)
{
	if(color){
		if(r)
			*r = (guchar)(color->r * 255.);
		if(g)
			*g = (guchar)(color->g * 255.);
		if(b)
			*b = (guchar)(color->b * 255.);
		if(a)
			*a = (guchar)(color->a * 255.);
	}
}

void color_parse_str(SCColor *sccolor, const gchar *spec)
{
	g_return_if_fail(sccolor != NULL && spec != NULL);
#if GTK_CHECK_VERSION(3, 4, 0)
	GdkRGBA rgba;

	rgba.red   = sccolor->r;
	rgba.green = sccolor->g;
	rgba.blue  = sccolor->b;
	rgba.alpha = sccolor->a;

	gdk_rgba_parse(&rgba, spec);

	sccolor->r = rgba.red;
	sccolor->g = rgba.green;
	sccolor->b = rgba.blue;
	sccolor->a = rgba.alpha;
#else
	GdkColor color;

	gdk_color_parse(spec, &color);
	sccolor->r  = (double)color.red   / 65535.;
	sccolor->g  = (double)color.green / 65535.;
	sccolor->b  = (double)color.blue  / 65535.;
	sccolor->a  = 1.;
#endif
}

void color_button_get_sccolor(GtkColorButton *button, SCColor *color)
{
	g_return_if_fail(GTK_IS_COLOR_BUTTON(button));
	g_return_if_fail(color != NULL);

#if GTK_CHECK_VERSION(3, 4, 0)
		GdkRGBA rgba = {0};

		gtk_color_button_get_rgba(button, &rgba);

		color->r = rgba.red;
		color->g = rgba.green;
		color->b = rgba.blue;
		color->a = rgba.alpha;
#else
		GdkColor gdkcolor;
		guint16 alpha;

		alpha = gtk_color_button_get_alpha(GTK_COLOR_BUTTON(button));
		gtk_color_button_get_color(GTK_COLOR_BUTTON(button), &gdkcolor);

		color->r = (double)gdkcolor.red   / 65535.;
		color->g = (double)gdkcolor.green / 65535.;
		color->b = (double)gdkcolor.blue  / 65535.;
		color->a = (double)alpha          / 65535.;
#endif
}

static void color_get_color_button_sccolor(char *btn_name, SCColor *color)
{
	GtkWidget *button = get_widget(btn_name);

	color_button_get_sccolor(GTK_COLOR_BUTTON(button), color);
}

#if !GTK_CHECK_VERSION(3, 4, 0)
static int sccolor_to_gdkcolor(SCColor *sccolor, GdkColor *color, guint16 *alpha)
{
	if(color && sccolor){

		color->red    = sccolor->r * 65535.;
		color->green  = sccolor->g * 65535.;
		color->blue   = sccolor->b * 65535.;

		if(alpha)
			*alpha    = sccolor->a * 65535.;

		return 1;
	}

	return 0;
}
#endif

static
void color_button_color_set(char *btn_name, SCColor *sccolor)
{
	GtkWidget *btn = get_widget(btn_name);

	if(!btn) return;

#if GTK_CHECK_VERSION(3, 4, 0)
	GdkRGB color;

	color->red    = sccolor->r;
	color->green  = sccolor->g;
	color->blue   = sccolor->b;
	color->alpha   = sccolor->a;

	gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(btn), &color);

#else
	/* Alpha must be set 1st. Race condition */
	if(sccolor){
		GdkColor color;
		guint16 alpha;

		sccolor_to_gdkcolor(sccolor, &color, &alpha);

		gtk_color_button_set_alpha(GTK_COLOR_BUTTON(btn), alpha);
		gtk_color_button_set_color(GTK_COLOR_BUTTON(btn), &color);
	}
#endif
}

// Notify fg SCColor changed
static void button_fg_notify_color(GtkColorButton *btn, GParamSpec *pspec)
{
	SCColor color;

	color_get_color_button_sccolor("colorbutton_fg", &color);
	dibuja_set_fg_sccolor(&color);
}

// Notify bg SCColor changed
static void button_bg_notify_color(GtkColorButton *btn, GParamSpec *pspec)
{
	SCColor color;

	color_get_color_button_sccolor("colorbutton_bg", &color);
	dibuja_set_bg_sccolor(&color);
}

static GtkWidget *create_button(const char *colordesc, char *name)
{
	GtkWidget *pal_color = palette_color_new(colordesc);

	g_object_set_data(G_OBJECT(dibuja_get_canvas()), name, (gpointer)pal_color);
	g_signal_connect(G_OBJECT(pal_color), "color-selected",
							 G_CALLBACK(tool_color_button_press_event), NULL);

	return pal_color;
}

void color_init(GtkBuilder *builder, double fg[4], double bg[4])
{
	GtkWidget *color_button_fg;
	GtkWidget *color_button_bg;
	GtkWidget *button, *table, *child;
	char name[40];
	int i = 0;
	int left, top;
	char *tooltip = 
					N_("Left click to set this color\n"
					"as the foreground color.\n\n"
					"Right click to set this color\n"
					"as the background color.");

	color_button_fg = get_widget("colorbutton_fg");
	g_signal_connect(G_OBJECT(color_button_fg), NOTIFY_COLOR,
					 G_CALLBACK(button_fg_notify_color), NULL);

	color_button_bg = get_widget("colorbutton_bg");
	g_signal_connect(G_OBJECT(color_button_bg), NOTIFY_COLOR,
					 G_CALLBACK(button_bg_notify_color), NULL);

	table = GTK_WIDGET (gtk_builder_get_object (builder, "table_col_pal"));

	gtk_table_resize(GTK_TABLE(table), 12, 2);
	for(top = 2; top < 12; top++){
		for(left = 0; left < 2; left++){
			sprintf(name, "button%d", i);
			button = create_button(default_colors[i], name);
			child = palette_color_get_child(PALETTE_COLOR(button));
			gtk_widget_set_tooltip_text(child, _(tooltip));

			gtk_table_attach (GTK_TABLE (table), button,
							  left, left + 1,
							  top, top + 1,
							  GTK_FILL, GTK_FILL, 0, 0);
			i++;
		}
	}

	{   /*******/
		SCColor fgcolor = {fg[0], fg[1], fg[2], fg[3]};
		SCColor bgcolor = {bg[0], bg[1], bg[2], bg[3]};
		
		color_button_color_set("colorbutton_fg", &fgcolor);
		color_button_color_set("colorbutton_bg", &bgcolor);
	}   /*******/

}

// Palette color clicked
static void tool_color_button_press_event(PaletteColor *pc, gint button)
{
	SCColor color = {0};

	palette_color_get_color(pc, &color);

	if(MOUSE_LEFT_BUTTON == button){
		color_button_color_set("colorbutton_fg", &color);
	}
	else if(MOUSE_RIGHT_BUTTON == button){
		color_button_color_set("colorbutton_bg", &color);
	}
}

// Switch the colors of the color buttons
gboolean colorbutton_right_click(GtkWidget *colorbutton, GdkEventButton *event, gpointer data)
{
	if(MOUSE_RIGHT_BUTTON == event->button){
		SCColor fg, bg;

		color_get_color_button_sccolor("colorbutton_fg", &fg);
		color_get_color_button_sccolor("colorbutton_bg", &bg);

		color_button_color_set("colorbutton_fg", &bg);
		color_button_color_set("colorbutton_bg", &fg);
	}
	return FALSE;
}


/* TODO */
/*static int color_edit_dialog_set_palette(gchar *palette)
{
	if(!palette) return 0;

	return 1;
}*/

/* Has default color palette on startup */
void color_edit_dialog(GtkWidget *widget)
{
	gint result;
	static GtkWidget *dialog = NULL;

	if(!GTK_IS_DIALOG(dialog)){
		dialog = dibuja_color_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(widget)));
	}

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	if(GTK_RESPONSE_ACCEPT == result){
		SCColor *colors = NULL;
		int i, ncolors;
		GtkWidget *button;
		char name[40];
		GtkWidget *canvas = dibuja_get_canvas();

		colors = dibuja_colorsel_get_colors(&ncolors);

		if(colors){
			for(i = 0; i < ncolors; i++){
				sprintf(name, "button%d", i);
				button = GTK_WIDGET (g_object_get_data(G_OBJECT(canvas), name));
				if(PALETTE_IS_COLOR(button)){
					palette_color_set_color(PALETTE_COLOR(button), &colors[i]);
				}
			}
			g_free(colors);
		}
	}
	gtk_widget_hide(dialog);
}



