/*
 * (c) 2015 - 2020 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "dibuja-colorsel.h"
#include "util.h"


#define NROWS	(11) /* 10 dareas, 1 pushbutton */
#define NCOLS	(2)
#define NCOLORS	(20)


static void set_dnd_src(void);
static void reset_palette_clicked(GtkWidget *button);

static gchar tooltip[] =
					N_("Click this palette entry to make it the current "
					"color. To change this entry, drag a color swatch "
					"here or right-click it and select \"Save color "
					"here.\"");
static GtkWidget *dareas[10][2];
static const GtkTargetEntry targets[] = {
								{ "application/x-color", 0 }
							};


static const gchar *default_colors[NCOLORS] = {
				"black", "white", "gray50", "red", "purple",
				"blue", "light blue", "green", "yellow", "orange",
				"lavender", "brown", "goldenrod4", "dodger blue", "pink",
				"light green", "gray10", "gray30", "gray75", "gray90"
			};

static SCColor *get_color(GtkWidget *darea)
{
	return (SCColor *)g_object_get_data(G_OBJECT(darea), "color");
}

static void
dibuja_color_selection_set_color(GtkWidget *colorsel, SCColor *color)
{
	GdkColor gdkcol = {0};

	if(!color) return;

	gdkcol.red   = (guint16)(color->r * 65535.);
	gdkcol.green = (guint16)(color->g * 65535.);
	gdkcol.blue  = (guint16)(color->b * 65535.);

	gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(colorsel),
										  &gdkcol);
	gtk_color_selection_set_current_alpha(GTK_COLOR_SELECTION(colorsel),
										  (guint16)(color->a * 65535.));
}

static GtkWidget *dibuja_color_selection_new(void)
{
	return gtk_color_selection_new();
}

static void
dibuja_color_selection_set_has_opacity_control(GtkWidget *cs, gboolean has_op)
{
	gtk_color_selection_set_has_opacity_control(GTK_COLOR_SELECTION(cs),
												has_op);
}

// TODO: Remove 'alpha' variable
static void set_color_from_rgba(GtkWidget *darea,
								guint16 r, guint16 g, guint16 b, guint16 a)
{
	SCColor *color = NULL;

	color = (SCColor *)g_object_get_data(G_OBJECT(darea), "color");
	if(color){
		color->r = r / 65535.;
		color->g = g / 65535.;
		color->b = b / 65535.;
		color->a = a / 65535.;
	}
}

static void darea_data_get(GtkWidget *darea, GdkDragContext *context,
						GtkSelectionData *selection_data,
						guint target_type, guint time, gpointer user_data)
{
	guint16 rgba[4];
	SCColor *color = NULL;

	color = get_color(darea);
	if(!color){
		rgba[0] = 1.;
		rgba[1] = 1.;
		rgba[2] = 1.;
		rgba[3] = 1.;
	}
	else{
		rgba[0] = color->r * 65535.;
		rgba[1] = color->g * 65535.;
		rgba[2] = color->b * 65535.;
		rgba[3] = color->a * 65535.;
	}

	gtk_selection_data_set (selection_data,
			  gdk_atom_intern_static_string ("application/x-color"),
			  16, (guchar *)rgba, 8);
}

static void
darea_drag_data_recieved(GtkWidget *darea, GdkDragContext *drag_context,
						 gint x, gint y, GtkSelectionData *seldata, guint info,
						 guint time, gpointer user_data)
{
	char *str = NULL;
	GdkAtom atom;
	guint16 *rgba = NULL;

	atom = gtk_selection_data_get_data_type(seldata);
	str = gdk_atom_name(atom);

	g_free(str);
	rgba = (guint16 *)gtk_selection_data_get_data(seldata);
	if(rgba){
		set_color_from_rgba(darea, rgba[0], rgba[1], rgba[2], rgba[3]);
		gtk_widget_queue_draw(darea);
	}
	else{
		g_message("[%s: %d] ERROR: dest_drag_data_recieved NULL data  %p\n",
						__FILE__, __LINE__, darea);
	}
	gtk_drag_finish(drag_context, TRUE, FALSE, time);
}

static gboolean darea_expose(GtkWidget *darea, GdkEventExpose *event, gpointer data)
{
	cairo_t *cr;
	SCColor *color = NULL;
	SCColor c2 = {1., 0., 0., 1.};

	color = (SCColor *)g_object_get_data(G_OBJECT(darea), "color");
	if(!color){
		color = &c2;
	}

	cr = gdk_cairo_create(gtk_widget_get_window(darea));
	cairo_set_source_rgba(cr, color->r, color->g, color->b, color->a);
	gdk_cairo_rectangle(cr, &(event->area));
	cairo_paint(cr);
	cairo_destroy(cr);

	return TRUE;
}
// TODO: Color selection
static gboolean darea_button_release(GtkWidget *darea, GdkEventButton *event,
							  GtkColorSelection *colorsel)
{
	SCColor *color = get_color(darea);

	if(!GTK_IS_COLOR_SELECTION(colorsel)){
		return TRUE;
	}

	if(color){
		dibuja_color_selection_set_color(GTK_WIDGET(colorsel), color);
		gtk_widget_queue_draw(GTK_WIDGET(colorsel));
	}

	return TRUE;
}

static GtkWidget *make_color_button(SCColor clr, GtkColorSelection *colorsel)
{
	GtkWidget *darea = gtk_drawing_area_new();
	SCColor *color = NULL;
	gint events = GDK_ALL_EVENTS_MASK;
	GtkDestDefaults dest_flags = GTK_DEST_DEFAULT_HIGHLIGHT |
								 GTK_DEST_DEFAULT_MOTION |
								 GTK_DEST_DEFAULT_DROP;

	gtk_widget_set_events(darea, events);
	color = g_malloc(sizeof(SCColor));
	*color = clr;
	g_object_set_data_full(G_OBJECT(darea), "color", color, g_free);
	g_signal_connect(G_OBJECT(darea), "expose-event",
					 G_CALLBACK(darea_expose), NULL);
	g_signal_connect(G_OBJECT(darea), "button-release-event",
					 G_CALLBACK(darea_button_release), colorsel);

	// Setup dnd recieve
	gtk_drag_dest_set (darea, dest_flags, targets, 1, GDK_ACTION_COPY);
	g_signal_connect(G_OBJECT(darea), "drag-data-received",
									G_CALLBACK(darea_drag_data_recieved), NULL);

	gtk_widget_set_size_request(darea, 20, 20);
	gtk_widget_set_tooltip_text(darea, _(tooltip));
	gtk_widget_show(darea);

	return darea;
}
/* TODO: store dareas per table */
GtkWidget *dibuja_color_dialog_new(GtkWindow *window)
{
	int left, top, cnt = 0;
	GtkWidget *content_area, *hbox, *widget, *table, *colsel;
	GtkWidget *dialog = GTK_WIDGET(
							gtk_dialog_new_with_buttons ("Edit Palette", window,
											GTK_DIALOG_DESTROY_WITH_PARENT,
											GTK_STOCK_CANCEL,
											GTK_RESPONSE_CANCEL,
											GTK_STOCK_APPLY,
											GTK_RESPONSE_ACCEPT,
											NULL)
							);

	gtk_window_set_icon_from_file(GTK_WINDOW(dialog),
								  DIBUJA_LOGO,
								  NULL);
	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
	hbox = gtk_hbox_new(FALSE, 10);
	gtk_container_add (GTK_CONTAINER (content_area), hbox);
	colsel = dibuja_color_selection_new();
	dibuja_color_selection_set_has_opacity_control(colsel, TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), colsel, FALSE, TRUE, 0);
	table = gtk_table_new(NROWS, NCOLS, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, TRUE, 0);

	widget = gtk_button_new();
	g_signal_connect(G_OBJECT(widget), "clicked",
					 G_CALLBACK(reset_palette_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(widget),
						gtk_image_new_from_stock(GTK_STOCK_REFRESH,
												GTK_ICON_SIZE_SMALL_TOOLBAR));
	gtk_widget_set_tooltip_text(widget, _("Click to set the palette to it's "
										"default colors."));

	gtk_table_attach(GTK_TABLE(table),
							 widget,
							 0, 2,
							 0, 1,
							 GTK_FILL, GTK_FILL,
							 0, 0);

	cnt = 0;
	for(top = 1; top < NROWS; top++){
		for(left = 0; left < NCOLS; left++){
			SCColor color = {0., 0., 0., 1.};

			widget = gtk_frame_new(NULL);
			gtk_frame_set_shadow_type(GTK_FRAME(widget), GTK_SHADOW_IN);
			color_parse_str(&color, default_colors[cnt]);
			cnt++;
			gtk_container_add(GTK_CONTAINER (widget),
							  make_color_button(color,
							  GTK_COLOR_SELECTION(colsel)));

			dareas[top - 1][left] = gtk_bin_get_child(GTK_BIN(widget));
			gtk_table_attach(GTK_TABLE(table),
							 widget,
							 left, left + 1,
							 top, top + 1,
							 GTK_FILL, GTK_FILL,
							 0, 0);
		}
	}

	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
	gtk_widget_show_all(dialog);
	/* Set here cause dnd send needs GdkWindow  */
	set_dnd_src();

	return dialog;
}

static void set_dnd_src(void)
{
 	int row, col;
 	GtkWidget *darea;
 	gint events = GDK_ALL_EVENTS_MASK;

 	for(row = 0; row < 10; row++){
		for(col = 0; col < 2; col++){

			darea = dareas[row][col];

			gtk_widget_add_events(darea, events);
			// Setup dnd send
			g_signal_connect (darea, "drag-data-get", G_CALLBACK (darea_data_get), NULL);
			gtk_drag_source_set(darea, GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
									   targets, G_N_ELEMENTS(targets),
									   GDK_ACTION_COPY | GDK_ACTION_MOVE);
		}
	}
}

static void reset_palette_clicked(GtkWidget *button)
{
	GtkWidget *darea;
	SCColor *color = NULL;
	int cnt = 0;
	int row, col;

	for(row = 0; row < 10; row++){
		for(col = 0; col < 2; col++){

			darea = dareas[row][col];
			color = (SCColor *)g_object_get_data(G_OBJECT(darea), "color");
			if(color){
				color_parse_str(color, default_colors[cnt]);
			}
			else
				g_warning("[%s - %s(): %d] Invalid color\n",
						  __FILE__, __func__, __LINE__);
			gtk_widget_queue_draw(darea);
			cnt++;
		}
	}
}

SCColor *dibuja_colorsel_get_colors(int *ncolors)
{
	GtkWidget *darea;
	int cnt = 0;
	int row, col;
	SCColor *pal_colors = NULL;
	SCColor *pcol = NULL;

	pal_colors = g_malloc(sizeof(SCColor) * NCOLORS);
	if(!pal_colors){
		if(ncolors){
			*ncolors = 0;
			return NULL;
		}
	}
	memset(pal_colors, 0xFF, sizeof(SCColor) * NCOLORS);

	for(row = 0; row < 10; row++){
		for(col = 0; col < 2; col++){

			darea = dareas[row][col];
			pcol = get_color(darea);
			if(pcol){
				pal_colors[cnt] = *pcol;
				cnt++;
			}
		}
	}

	if(ncolors) *ncolors = cnt;

	return pal_colors;
}

