/*
 * palette-color.c
 * 
 * Copyright 2018 - 2020 Juan Balderas
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "palette-color.h"

#define PALETTE_ENTRY_WIDTH		(25)
#define PALETTE_ENTRY_HEIGHT	(25)


struct _PaletteColorPrivate
{
  SCColor color;
};

enum {
	COLOR_SELECTED,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

static void palette_color_finalize (GObject *object);
static gboolean palette_color_expose(GtkWidget *widget, GdkEventExpose *event,
									 gpointer data);
static gboolean palette_color_draw(GtkWidget *widget, cairo_t *cr,
								   gpointer data);
static gboolean
palette_color_button_press_event(GtkWidget *widget, GdkEventButton *event,
							  gpointer data);

G_DEFINE_TYPE_WITH_CODE (PaletteColor, palette_color, GTK_TYPE_FRAME, G_ADD_PRIVATE(PaletteColor))



static 
void create_button(PaletteColor *pc, const char *colordesc)
{
	GtkWidget *darea = NULL;
	GtkWidget *frame = GTK_WIDGET(pc);

	gtk_widget_set_size_request(frame, PALETTE_ENTRY_WIDTH,
								PALETTE_ENTRY_HEIGHT);

	darea = gtk_drawing_area_new();
	gtk_container_add (GTK_CONTAINER (frame), darea);
	gtk_widget_set_events(darea, GDK_BUTTON_PRESS_MASK |
						  GDK_EXPOSURE_MASK);

	color_parse_str(&(pc->priv->color), colordesc);

#if GTK_MAJOR_VERSION >= 3
	gtk_frame_set_shadow_type(GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
	g_signal_connect(G_OBJECT(darea), "draw",
					 G_CALLBACK(palette_color_draw), (gpointer)pc);
#else
	gtk_frame_set_shadow_type(GTK_FRAME (frame), GTK_SHADOW_OUT);
	g_signal_connect(G_OBJECT(darea), "expose-event",
					 G_CALLBACK(palette_color_expose), (gpointer)pc);
#endif
	g_signal_connect(G_OBJECT(darea), "button-press-event",
					 G_CALLBACK(palette_color_button_press_event), (gpointer)pc);

}

static void
palette_color_class_init (PaletteColorClass *klass)
{
  GObjectClass *g_object_class;
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

  g_object_class = G_OBJECT_CLASS (klass);

  g_object_class->finalize = palette_color_finalize;

  signals[COLOR_SELECTED] =
    g_signal_new ("color-selected",
		  G_OBJECT_CLASS_TYPE (widget_class),
		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
		  G_STRUCT_OFFSET (PaletteColorClass, color_selected),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__OBJECT,
		  G_TYPE_NONE, 1,
		  G_TYPE_INT);
}


static void
palette_color_finalize (GObject *object)
{
  g_return_if_fail (PALETTE_IS_COLOR (object));

  G_OBJECT_CLASS (palette_color_parent_class)->finalize (object);
}


static void
palette_color_init (PaletteColor *self)
{
  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PALETTE_TYPE_COLOR, PaletteColorPrivate);
}

static
gboolean palette_color_draw(GtkWidget *widget, cairo_t *cr, gpointer data)
{
	SCColor color;
	GtkAllocation allocation;
	PaletteColor *pc = PALETTE_COLOR(data);

	palette_color_get_color(pc, &color);
	cairo_set_source_rgba(cr, color.r, color.g, color.b, color.a);
	gtk_widget_get_allocation(widget, &allocation);
	cairo_rectangle(cr, 0., 0., allocation.width, allocation.height);
	cairo_fill(cr);

	return TRUE;
}

#if GTK_MAJOR_VERSION  < 3
static gboolean palette_color_expose(GtkWidget *widget, GdkEventExpose *event,
									 gpointer data)
{
	if(gtk_widget_get_visible(widget) && gtk_widget_get_mapped(widget)){
		cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
		palette_color_draw(widget, cr, data);
		cairo_destroy(cr);
	}

	return TRUE;
}
#endif

static gboolean
palette_color_button_press_event(GtkWidget *widget, GdkEventButton *event,
									   gpointer data)
{
	PaletteColor *pc = PALETTE_COLOR(data);

	g_signal_emit (pc, signals[COLOR_SELECTED], 0, event->button);

	return TRUE;
}

GtkWidget *palette_color_new (const char *colordesc)
{
	PaletteColor *pc = PALETTE_COLOR(g_object_new (PALETTE_TYPE_COLOR, NULL));

	create_button(pc, colordesc);

	return GTK_WIDGET(pc);
}

void palette_color_get_color(PaletteColor *pc, SCColor *sccolor)
{
	g_return_if_fail(PALETTE_IS_COLOR(pc) && sccolor != NULL);

	*sccolor = pc->priv->color;
}

void palette_color_set_color(PaletteColor *pc, SCColor *color)
{
	g_return_if_fail(PALETTE_IS_COLOR(pc) && color != NULL);

	pc->priv->color = *color;

	gtk_widget_queue_draw(palette_color_get_child(pc));
}

GtkWidget *palette_color_get_child(PaletteColor *pc)
{
	g_return_val_if_fail(PALETTE_IS_COLOR(pc), NULL);

	return gtk_bin_get_child(GTK_BIN(pc));
}
